/**
 * @license
 *-------------------------------------------------------------------------------------------
 * Copyright © 2025 Progress Software Corporation. All rights reserved.
 * Licensed under commercial license. See LICENSE.md in the package root for more information
 *-------------------------------------------------------------------------------------------
 */
"use client";
import * as o from "react";
import e from "prop-types";
import { cloneDate as f, isEqual as L } from "@progress/kendo-date-math";
import { Button as x } from "@progress/kendo-react-buttons";
import { caretAltUpIcon as _, caretAltDownIcon as V } from "@progress/kendo-svg-icons";
import { provideLocalizationService as A, provideIntlService as P, registerForIntl as F, registerForLocalization as N } from "@progress/kendo-react-intl";
import { getActiveElement as M, validatePackage as q, classNames as u, uDateInput as m, createPropsContext as B, withIdHOC as R, withPropsContext as j, withUnstyledHOC as U } from "@progress/kendo-react-common";
import { FloatingLabel as z } from "@progress/kendo-react-labels";
import { DateInput as H } from "@progress/kendo-dateinputs-common";
import { DEFAULT_FORMAT as K, DEFAULT_FORMAT_PLACEHOLDER as W, isInRange as J } from "./utils.mjs";
import { packageMetadata as X } from "../package-metadata.mjs";
import { MAX_DATE as Y, MIN_DATE as G, MIN_TIME as Z, MAX_TIME as $ } from "../utils.mjs";
import { increaseValue as g, messages as v, decreaseValue as b } from "../messages/index.mjs";
import { isInTimeRange as Q } from "../timepicker/utils.mjs";
import ee from "../common/ClearButton.mjs";
import { DateInputIntl as te } from "./dateInputIntl.mjs";
const ie = "Please enter a valid value!", n = class n extends o.Component {
  constructor(t) {
    super(t), this.kendoDate = null, this.kendoDateObject = null, this.mounted = !1, this._element = null, this._wrapper = null, this._nativeEvent = null, this.focus = () => {
      this._element && this._element.focus();
    }, this.setValidity = () => {
      this.element && this.element.setCustomValidity && this.element.setCustomValidity(
        this.validity.valid ? "" : this.props.validationMessage || n.defaultProps.validationMessage
      );
    }, this.updateOnPaste = (i) => {
      !this.element || !this.kendoDate || this.triggerChange(i);
    }, this.spinnersMouseDown = (i) => {
      i.preventDefault();
      const s = M(document);
      this.element && s !== this.element && this.element.focus({ preventScroll: !0 });
    }, this.handleClick = (i) => {
      this.props.onChange && this.props.onChange(i);
    }, this.nativeWheel = (i) => {
      M(document) === this.element && i.preventDefault();
    }, this.increasePart = (i) => {
      var s;
      (s = this.kendoDate) == null || s.modifyDateSegmentValue(1), this.onChange(i);
    }, this.decreasePart = (i) => {
      var s;
      (s = this.kendoDate) == null || s.modifyDateSegmentValue(-1), this.onChange(i);
    }, this.elementOnFocus = (i) => {
      this.wrapper && this.wrapper.classList.add("k-focus"), this.setState({ focused: !0 });
    }, this.elementOnBlur = (i) => {
      this.wrapper && this.wrapper.classList.remove("k-focus"), this.setState({ focused: !1 });
    }, q(X), this.state = {
      focused: !1
    };
  }
  get _inputId() {
    return this.props.id + "-accessibility-id";
  }
  isFormatChanged(t, i) {
    return typeof t != typeof i ? !0 : typeof t == "string" && typeof i == "string" ? t !== i : typeof t == "object" && typeof i == "object" ? JSON.stringify(t) !== JSON.stringify(i) : !1;
  }
  mapFormatToCommonFormat(t) {
    return typeof t == "string" ? t : {
      inputFormat: t,
      displayFormat: t
    };
  }
  /* Public Getters */
  /**
   * Gets the value of the DateInput.
   */
  get value() {
    return this.valueDuringOnChange !== void 0 ? this.valueDuringOnChange : this.kendoDate && this.kendoDate.value;
  }
  /**
   * Gets the `name` property of the DateInput.
   */
  get name() {
    return this.props.name;
  }
  get min() {
    return this.props.min !== void 0 ? this.props.min : n.defaultProps.min;
  }
  get max() {
    return this.props.max !== void 0 ? this.props.max : n.defaultProps.max;
  }
  get isInSegmentSelection() {
    const { year: t, month: i, day: s, hours: a, minutes: r, seconds: l } = this.kendoDateObject;
    return (!t || !i || !s || !a || !r || !l) && (t || i || s || a || r || l);
  }
  /**
   * @hidden
   */
  get text() {
    const { placeholder: t } = this.props, i = this.kendoDate && this.kendoDate.currentText || "";
    return t != null && !this.state.focused && !this.value ? t : i;
  }
  /**
   * Represents the validity state into which the DateInput is set.
   */
  get validity() {
    const t = J(this.value, this.min, this.max) && Q(this.value, this.props.minTime, this.props.maxTime), i = this.props.validationMessage !== void 0, s = (!this.required || this.value !== null) && t, a = this.props.valid !== void 0 ? this.props.valid : s;
    return {
      customError: i,
      rangeOverflow: this.value && this.max.getTime() < this.value.getTime() || !1,
      rangeUnderflow: this.value && this.value.getTime() < this.min.getTime() || !1,
      valid: a,
      valueMissing: this.value === null
    };
  }
  /**
   * Gets the element of the DateInput.
   *
   * @return - An `HTMLInputElement`.
   *
   * @example
   * ```jsx
   * class App extends React.Component {
   *     constructor(props) {
   *         super(props);
   *     }
   *     element = null;
   *     render() {
   *         return (
   *             <div>
   *                 <DateInput
   *                     ref={(dateInput) =>
   *                         this.element = dateInput ? dateInput.element : null}
   *                 />
   *                 <button onClick={() => console.log(this.element)}>console.log the element</button>
   *             </div>
   *         );
   *     }
   * }
   *
   * ReactDOM.render(
   *     <App />,
   *     document.getElementsByTagName('my-app')[0]
   * );
   * ```
   */
  get element() {
    return this._element;
  }
  /**
   * @hidden
   */
  get validityStyles() {
    return this.props.validityStyles !== void 0 ? this.props.validityStyles : n.defaultProps.validityStyles;
  }
  /**
   * @hidden
   */
  get required() {
    return this.props.required !== void 0 ? this.props.required : !1;
  }
  /**
   * @hidden
   */
  get wrapper() {
    return this._wrapper;
  }
  /**
   * @hidden
   */
  get options() {
    const {
      format: t,
      steps: i,
      placeholder: s,
      value: a,
      autoFill: r,
      enableMouseWheel: l,
      autoCorrectParts: h,
      autoSwitchParts: p,
      autoSwitchKeys: y,
      twoDigitYearMax: D,
      allowCaretMode: d,
      formatPlaceholder: C
    } = this.props;
    return {
      format: t,
      steps: i,
      formatPlaceholder: C,
      placeholder: s,
      selectPreviousSegmentOnBackspace: !0,
      value: a || this.value,
      intlService: this.intlCommon,
      autoFill: r !== void 0 ? r : !1,
      enableMouseWheel: l,
      autoCorrectParts: h,
      autoSwitchParts: p,
      autoSwitchKeys: y || [],
      twoDigitYearMax: D,
      allowCaretMode: d
    };
  }
  /**
   * @hidden
   */
  componentDidMount() {
    this.setValidity(), this.mounted || (this.wrapper && this.wrapper.addEventListener("wheel", this.nativeWheel, { passive: !1 }), this.kendoDate = this.initKendoDate(), this.kendoDateObject = this.kendoDate.dateObject, this.mounted = !0);
  }
  /**
   * @hidden
   */
  componentDidUpdate(t) {
    this.setValidity(), this.kendoDate && ((this.isFormatChanged(t.format, this.props.format) || t.readonly !== this.props.readonly || t.steps !== this.props.steps || this.intlCommon.locale !== this.kendoDate.options.intlService.locale) && this.kendoDate.setOptions(this.options, !0), t.value !== this.props.value && (this.kendoDateObject.getValue() !== null || this.isInSegmentSelection && this.props.value !== null) && this.kendoDateObject.setValue(this.props.value), this.props.ariaExpanded !== void 0 && this.props.ariaExpanded && (this.kendoDate.options.placeholder = null), this.props.ariaExpanded !== void 0 && !this.props.ariaExpanded && (this.kendoDate.options.placeholder = this.props.placeholder), this.kendoDate.refreshElementValue());
  }
  /**
   * @hidden
   */
  componentWillUnmount() {
    this.wrapper && this.wrapper.removeEventListener("wheel", this.nativeWheel);
  }
  /**
   * @hidden
   */
  render() {
    var S;
    const {
      size: t = n.defaultProps.size,
      fillMode: i = n.defaultProps.fillMode,
      rounded: s = n.defaultProps.rounded,
      autoFocus: a = n.defaultProps.autoFocus,
      inputAttributes: r,
      unstyled: l
    } = this.props, h = l && l.uDateInput;
    this.props._ref && (this.props._ref.current = this);
    const p = A(this), y = { ...n.defaultProps, ...this.props }, { name: D, label: d, id: C, disabled: T } = y, E = this.text, k = C || this._inputId, O = !this.validityStyles || this.validity.valid, I = /* @__PURE__ */ o.createElement(
      "span",
      {
        ref: (w) => {
          this._wrapper = w;
        },
        style: d ? void 0 : { width: this.props.width },
        dir: this.props.dir,
        className: u(
          m.wrapper({
            c: h,
            size: t,
            fillMode: i,
            rounded: s,
            disabled: T,
            required: this.required,
            invalid: !O
          }),
          this.props.className
        )
      },
      /* @__PURE__ */ o.createElement(
        "input",
        {
          role: this.props.ariaRole || "textbox",
          readOnly: this.props.readonly,
          tabIndex: this.props.tabIndex || 0,
          disabled: this.props.disabled,
          title: this.props.title !== void 0 ? this.props.title : E,
          value: E,
          type: "text",
          spellCheck: !1,
          autoComplete: "off",
          autoCorrect: "off",
          autoFocus: a,
          className: u(m.inputInner({ c: h })),
          id: k,
          "aria-label": this.props.ariaLabel,
          "aria-labelledby": this.props.ariaLabelledBy,
          "aria-describedby": this.props.ariaDescribedBy,
          "aria-haspopup": this.props.ariaHasPopup,
          "aria-disabled": this.props.disabled,
          "aria-expanded": this.props.ariaExpanded,
          "aria-controls": this.props.ariaControls,
          "aria-required": this.props.required,
          "aria-invalid": !O,
          onKeyDown: this.onChange.bind(this),
          onChange: this.onChange.bind(this),
          onWheel: this.onChange.bind(this),
          onInput: this.onChange.bind(this),
          onClick: this.onChange.bind(this),
          name: D,
          ref: (w) => this._element = w,
          ...r
        }
      ),
      this.props.children,
      this.props.clearButton && this.props.value && /* @__PURE__ */ o.createElement(ee, { onClick: this.handleClick, key: "clearbutton" }),
      this.props.spinners && /* @__PURE__ */ o.createElement(
        "span",
        {
          className: u(m.inputSpinner({ c: h })),
          onMouseDown: this.spinnersMouseDown
        },
        /* @__PURE__ */ o.createElement(
          x,
          {
            tabIndex: -1,
            type: "button",
            rounded: null,
            className: u(m.spinnerIncrease({ c: h })),
            icon: "caret-alt-up",
            svgIcon: _,
            "aria-label": p.toLanguageString(g, v[g]),
            title: p.toLanguageString(g, v[g]),
            onClick: this.increasePart
          }
        ),
        /* @__PURE__ */ o.createElement(
          x,
          {
            tabIndex: -1,
            type: "button",
            rounded: null,
            className: u(m.spinnerDecrease({ c: h })),
            icon: "caret-alt-down",
            svgIcon: V,
            "aria-label": p.toLanguageString(b, v[b]),
            title: p.toLanguageString(b, v[b]),
            onClick: this.decreasePart
          }
        )
      )
    );
    return d ? /* @__PURE__ */ o.createElement(
      z,
      {
        label: d,
        editorId: k,
        editorValue: (S = this._element) == null ? void 0 : S.value,
        editorValid: O,
        editorDisabled: this.props.disabled,
        children: I,
        style: { width: this.props.width }
      }
    ) : I;
  }
  /**
   * @hidden
   */
  intl() {
    return P(this);
  }
  /**
   * @hidden
   */
  get intlCommon() {
    return new te(P(this));
  }
  /*  end handlers */
  triggerChange(t) {
    this.valueDuringOnChange = this.value, this.forceUpdate(), this._nativeEvent = t, this.valueDuringOnChange = void 0;
  }
  onChange(t) {
    this._nativeEvent && this.props.onChange && !L(this._nativeEvent.oldValue, this.value) && this.props.onChange.call(void 0, {
      syntheticEvent: t,
      nativeEvent: this._nativeEvent.event,
      value: this._nativeEvent.value,
      target: this
    }), this._nativeEvent = null;
  }
  initKendoDate() {
    return new H(this._element, {
      ...this.options,
      format: this.mapFormatToCommonFormat(this.options.format),
      events: {
        focus: this.elementOnFocus.bind(this),
        blur: this.elementOnBlur.bind(this),
        valueChange: this.triggerChange.bind(this),
        click: this.handleClick.bind(this)
      }
    });
  }
};
n.displayName = "DateInput", n.propTypes = {
  value: e.instanceOf(Date),
  format: e.oneOfType([
    e.string,
    e.shape({
      skeleton: e.string,
      pattern: e.string,
      date: e.oneOf(["short", "medium", "long", "full"]),
      time: e.oneOf(["short", "medium", "long", "full"]),
      datetime: e.oneOf(["short", "medium", "long", "full"]),
      era: e.oneOf(["narrow", "short", "long"]),
      year: e.oneOf(["numeric", "2-digit"]),
      month: e.oneOf(["numeric", "2-digit", "narrow", "short", "long"]),
      day: e.oneOf(["numeric", "2-digit"]),
      weekday: e.oneOf(["narrow", "short", "long"]),
      hour: e.oneOf(["numeric", "2-digit"]),
      hour12: e.bool,
      minute: e.oneOf(["numeric", "2-digit"]),
      second: e.oneOf(["numeric", "2-digit"]),
      timeZoneName: e.oneOf(["short", "long"])
    })
  ]),
  formatPlaceholder: e.oneOfType([
    e.oneOf([
      "wide",
      "narrow",
      "short",
      "formatPattern"
    ]),
    e.shape({
      year: e.string,
      month: e.string,
      day: e.string,
      hour: e.string,
      minute: e.string,
      second: e.string
    })
  ]),
  width: e.oneOfType([e.string, e.number]),
  tabIndex: e.number,
  title: e.string,
  steps: e.shape({
    year: e.number,
    month: e.number,
    day: e.number,
    hour: e.number,
    minute: e.number,
    second: e.number
  }),
  min: e.instanceOf(Date),
  max: e.instanceOf(Date),
  disabled: e.bool,
  spinners: e.bool,
  name: e.string,
  dir: e.string,
  label: e.string,
  id: e.string,
  ariaLabelledBy: e.string,
  ariaDescribedBy: e.string,
  ariaLabel: e.string,
  ariaRole: e.string,
  ariaHasPopup: e.oneOfType([e.bool, e.string]),
  ariaExpanded: e.oneOfType([e.bool]),
  onChange: e.func,
  validationMessage: e.string,
  required: e.bool,
  validate: e.bool,
  valid: e.bool,
  size: e.oneOf([null, "small", "medium", "large"]),
  rounded: e.oneOf([null, "small", "medium", "large", "full"]),
  fillMode: e.oneOf([null, "solid", "flat", "outline"]),
  autoFocus: e.bool,
  inputAttributes: e.object
}, n.defaultProps = {
  format: K,
  size: "medium",
  rounded: "medium",
  fillMode: "solid",
  formatPlaceholder: W,
  spinners: !1,
  disabled: !1,
  max: f(Y),
  min: f(G),
  minTime: f(Z),
  maxTime: f($),
  validityStyles: !0,
  validationMessage: ie,
  placeholder: null,
  enableMouseWheel: !0,
  autoCorrectParts: !0,
  autoSwitchParts: !0,
  allowCaretMode: !1,
  twoDigitYearMax: 68,
  // the rest of the properties are undefined by default
  ariaHasPopup: "grid",
  autoFocus: !1
};
let c = n;
const se = B(), ne = R(
  j(
    se,
    U(c)
  )
);
ne.displayName = "KendoReactDateInput";
F(c);
N(c);
export {
  ne as DateInput,
  se as DateInputPropsContext,
  c as DateInputWithoutContext
};
